dcuhre <- function( f, low, hi, ..., name="x", epsabs=0.0, epsrel=1.0e-4,
	mincall=0, maxcall=10000, key=0 ){
#**********************************************************************
#
#                       Integrate an S Function
# Using Algorithm 698 of toms by Jarle Berntsen and Terje O. Espelid
#   
# Function - Numerically integrates an S function f from low to hi.
#
# Arguments
#
#  f : function to be integrated. f must be a function of at least
#      one variable (defined by name) over which it will be
#      integrated. The variable of integration (call it x) must be a
#      double precision vector giving the coordinates of the evaluation
#      point. f must return a single variable which is a double
#      precision vector giving the values of the vector integrand at
#      the point x.
#      Let NDIM = length(x) and NUMFUN = length(f(x)) then
#      1 < NDIM <= 15
#      NUMFUN > 1
#
#  low : low value of integration. Must be of lenth NDIM
#
#  hi : high value of integration. Must be of length NDIM
#
#  name : a character string containing the name of the variable 
#         of integration
#
#  epsabs : bound on absolute error of integral
#
#  epsrel : bound on relative error of integral
#
#  mincall : Minimum number of function calls required.
#
#  maxcall : Maximum number of function calls the code is allowed to use.
#
#  key : Select the integration rule. The table below is of the
#        polynomial degress of the different integration rules
#        depending on the value of key and NDIM.
#
#        KEY   NDIM = 2   NDIM = 3   NDIM > 3
#         0      13         11         9
#         1      13         --        --
#         2      --         11        --
#         3       9          9         9
#         4       7          7         7
#
# ... : Any additional arguments to the function, f.
#
# Output
#
#  returns a list of
#
#  result : A vector of length NUMFUN containing approximations to all
#           components of the vector integral
#
#  abserr : A vector of length NUMFUN containing estimates for absalute
#           errors for all components of the vector integral
#
#  neval  : The number of evaluations used
#
# Method
#
#  Uses Toms Algorithm 698 by Berntse, Espelid and Genz
#
#**********************************************************************
#
# Error Checks on Input
if ( missing(f) || !is.function(f) )
   stop( "Argument `f' must be a function")
if ( missing(low) || !is.numeric(low) )
   stop( "Argument `low' must be numeric (low bnd for int)")
ndim <- length(low)
if ((ndim < 2) || (ndim > 15))
   stop ("DCUHRE only works for 2 to 15 variables in the integrand(s)")
if ( missing(hi) || !is.numeric(hi) )
   stop( "Argument `hi' must be numeric (hi bnd for int)")
if (length(hi) != ndim)
   stop( "Aguments `low' and `hi' must be the same length")
if (!is.numeric(epsabs) )
   stop( "Argument `epsabs' must be numeric")
if (!is.numeric(epsrel) )
   stop( "Arguement `epsrel' must be numeric")
if ( ( epsabs <= 0 ) && ( epsrel <= 0 ) )
   stop( 'error bound from epsabs and epsrel too small' )
if (!is.numeric(mincall))
   stop ("Arguement `mincall' must be numeric")
if (!is.numeric(maxcall) || (maxcall < mincall) || (maxcall < 0))
   stop ("Arguement `maxcall' must be a positive number > `mincall'")
if (!is.numeric(key) || (key < 0) || (key > 4))
   stop ("Invalid input for `key'. Should be a number between 0 and 4")
if ((key == 1) & (ndim != 2))
   stop ("Invalid input for `key'. key = 1 and ndim != 2")
if ((key == 2) & (ndim != 3))
   stop ("Invalid input for `key'. key = 2 and ndim != 3")
if ( !is.character(name))
   stop ("Argument `name' should be a character string")
#
#  Create, f2, a function of the single variable of integration
#
f.extra <- list(...)
f2 <- integrate.f.define(f,name,f.extra)
#
#  Call function, f2, with values in low and check output
#
fn.check <- f2(low)
numfun <- length(fn.check)
if (numfun < 1)
   stop ("function, `f' must return a vector of at least length 1")
#
#  Invoke C driver
#
nw <- 5000
integ.reg <- .C("dcucall",
		as.integer(ndim),
		as.integer(numfun),
		as.double(c(low)),
		as.double(c(hi)),
		as.integer(mincall),
		as.integer(maxcall),
		list(f2),
		as.double(epsabs),
		as.double(epsrel),
		as.integer(key),
		as.integer(nw),
		as.integer(0),
		result=as.double(rep(0,numfun)),
		abserr=as.double(rep(0,numfun)),
		neval=as.integer(0),
		ifail=as.integer(0),
		as.double(rep(0,nw)))
#
#  Check ifail for warning or error conditions
#
ifail <- integ.reg$ifail
if (ifail != 0) {
   if (ifail == 1)
      warning("Arguement `maxcall' too small to obtain requested error.")
   else if (ifail == 7) 
      stop ("The volume of the region of integration is zero.")
   else if (ifail == 8)
      stop ("Arguement `maxcall' too small.")
   else if (ifail == 11)
      stop ("NW too small (increase value of line 109 of dcuhre.s")
   else stop("Unable to integrate.")
}
#
# return value of integral
#
result <- integ.reg$result
abserr <- integ.reg$abserr
neval <- integ.reg$neval
return(result,abserr,neval)
}
gkint <- function( f, low, hi, ..., name="x", epsabs=1.0e-10, 
		       epsrel=1.0e-6 ){
#**********************************************************************
#
#                       Integrate an S Function
#   
# Function - Numerically integrates an S function f from low to hi.
#
# Arguments
#
#  f : function to be integrated
#
#  low : low value of integration
#
#  hi : high value of integration
#
#  name : a character string containing the name of the variable 
#         of integration
#
#  epsabs : bound on absolute error of integral
#
#  epsrel : bound on relative error of integral
#
# ... : Any additional arguments to the function, f.
#
# Output
#
#  returns a list with
#  result : estimated value of the integral
#
#  abserr : upper bound on absolute error of the integral
#
#  neval : number of function evaluations used
#
#
# Method
#
#  Uses dqag from quadpack.
#
#**********************************************************************
#
# check type of input arguments
#
if ( missing(f) || !is.function(f) )
   stop( "Argument `f' must be a function")
if ( missing(low) || !is.numeric(low))
   stop( "Argument `low' must be number (low bnd for int)")
if ( missing(hi) || !is.numeric(hi))
   stop( "Argument `hi' must be number (hi bnd for int)" )
if (!is.numeric(epsabs) )
   stop( "Argument `epsabs' must be numeric" ) 
if (!is.numeric(epsrel) )
   stop( "Argument `epsrel' must be numeric" )
if ( !is.character(name))
   stop ("Argument `name' should be a character string")
#
# check epsabs and epsrel for legality
#
if ( ( epsabs <= 0 ) && ( epsrel <= 1.0e-12 ) )
   stop( "error bound from epsabs and epsrel too small" )
#
#  Create, f2, a function of the single variable of integration
#
f.extra <- list(...)
f2 <- integrate.f.define(f,name,f.extra)
#
# invoke c driver
#
integ.res <- .C("drdqag",
                list(f2),
                as.double(low),
                as.double(hi),
                as.double(epsabs),
                as.double(epsrel),
                integral=double(length=1),
                abserr=double(length=1),
                neval=integer(length=1),
                ier=integer(length=1),
                as.integer(100),
                as.integer(400),
                integer(length=1),
                integer(length=100),
                double(length=400) 
                                    )
#
# issue warning messages for ier
#
  if ( integ.res$ier == 1)
      warn("maximum number of subdivisions has been acheived")
  if ( integ.res$ier == 2 )
     warn( "roundoff error may make result unreliable" )
  if ( integ.res$ier == 3 )
     warn(" bad integrand behavior makes result suspect" )
  if ( integ.res$ier == 6 )
     stop(" bad input values - should not happen" )
#
# Return output
#
  result <- as.double( integ.res$integral )
  abserr <- as.double(integ.res$abserr)
  neval <- as.integer(integ.res$neval)
  return(result,abserr,neval)
}
integrate.f.define <- function(f,name,f.extra)
{
#
# Assign Variables in f.extra to frame 1
n.extra <- length(f.extra)
call.extra <- ""
if (n.extra > 0) {
  names.extra <- names(f.extra)
  for (i in 1:n.extra) {
    temp <- paste("integrate.param.",i,sep="")
    call.extra <- paste(call.extra,",",names.extra[i],"=",temp,sep="")
    assign(temp,f.extra[[i]],frame=1)
  }
}
#
# Assign function f to frame 1
call.func <- "integrate.f"
assign(call.func,f,frame=1)
#
# Find parameters of the function, f
f.param <- names(f)
f.param <- f.param[f.param != ""]
#
# Make sure that the integration variable name is a parameter of f
if (!any(f.param == name))
   stop(paste(name,"is not a parameter of f"))
#
# Define a function call to f and assign it to frame 1
f.call <- paste(call.func,"(",name,"=x9999.z1111.q13",call.extra,sep="")
f.call <- paste(f.call,")",sep="")
assign("integrate.f.call",f.call,frame=1)
#
# Define a new function, f2, with a single variable
f2 <- function(x9999.z1111.q13) {
   ans <- as.double(eval(parse(text=integrate.f.call)))
   if ((!is.numeric(ans)) || any(is.na(ans)) || any(is.inf(ans)))
     stop("function, `f' must return a numeric value")
   return(ans)
}
return(f2)
}
intgauss <- function( f, low, hi, ..., name="x"){
# Integrates f from low to hi using a 10 point gaussian formula
#
if ( missing(f) || !is.function(f) )
   stop( "Argument `f' must be a function")
if ( missing(low) || !is.numeric(low))
   stop( "Argument `low' must be numeric (low bnd for int)")
if ( missing(hi) || !is.numeric(hi))
   stop( "Argument `hi' must be numeric (hi bnd for int)")
if ( !is.character(name))
   stop ("Argument `name' should be a character string")
#
#  Perform integration
#
x <- c(    0.148874338981631,0.433395394129247,
           0.679409568299024,0.865063366688985,0.973906528517172)
wt <- c(   0.295524224714753,0.269266719309996,
           0.219086362515982,0.149451349150581,0.066671344308688)
answer <- rep(NA,5)
mid <- ( low + hi ) / 2
h <- ( hi - low ) / 2
for ( i in 1:5) {
   f.call <- paste("f(",name,"=mid-h*x[i],...)",sep="")
   answer[i] <- eval(parse(text=f.call))
   f.call <- paste("f(",name,"=mid+h*x[i],...)",sep="")
   answer[i] <- answer[i] + eval(parse(text=f.call))
   answer[i] <- wt[i]*answer[i]
}
return( sum(answer) * h )       }
